@@ -40,6 +40,7 @@ L<http://www.chordpro.org>.
 use strict;
 use warnings;
 use Data::Dumper;
+use Carp;
 
 ################ The Process ################
 
@@ -84,6 +85,9 @@ sub main {
         elsif ( $of =~ /\.(crd|txt)$/i ) {
             $options->{generate} ||= "Text";
         }
+        elsif ( $of =~ /\.html?$/i ) {
+            $options->{generate} ||= "HTML";
+        }
         elsif ( $of =~ /\.(debug)$/i ) {
             $options->{generate} ||= "Debug";
         }
@@ -227,6 +231,15 @@ diagrams.
 Specify the encoding for input files. Default is UTF-8.
 ISO-8859.1 (Latin-1) encoding is automatically sensed.
 
+=item B<--filelist=>I<FILE>
+
+Read the names of the files to be processed from the named file.
+
+This option may be specified multiple times.
+
+Song file names listed on the command line are processed I<after> the
+files from the filelist arguments.
+
 =item B<--lyrics-only> (short: B<-l>)
 
 Only prints lyrics. All chords are suppressed.
@@ -617,13 +630,13 @@ sub app_setup {
           ### Options ###
 
           "output|o=s",                 # Saves the output to FILE
-          "lyrics-only",                # Suppress all chords
           "generate=s",
           "backend-option|bo=s\%",
 	  "diagrams=s",			# Prints chord diagrams
           "encoding=s",
 	  "csv!",			# Generates contents CSV
 	  "cover=s",			# Cover page(s)
+	  "filelist=s@",		# List of input files
 
           ### Standard Chordii Options ###
 
@@ -768,6 +781,20 @@ sub app_setup {
 	exit 0;
     }
 
+    if ( $clo->{filelist} ) {
+	my @files;
+	foreach ( @{ $clo->{filelist} } ) {
+	    my $list = ::loadfile( $_, $clo );
+	    foreach ( @$list ) {
+		next unless /\S/;
+		next if /^#/;
+		s/[\r\n]+$//;
+		push( @files, encode_utf8($_) );
+	    }
+	}
+	unshift( @ARGV, @files );
+    }
+
     # At this point, there should be filename argument(s)
     # unless we're embedded or just dumping chords.
     app_usage(\*STDERR, 1)
@@ -829,6 +856,7 @@ Options:
     --cover=FILE                  Add cover pages from PDF document
     --diagrams=WHICH		  Prints chord diagrams
     --encoding=ENC                Encoding for input files (UTF-8)
+    --filelist=FILE               Reads song file names from FILE
     --lyrics-only  -l             Only prints lyrics
     --output=FILE  -o             Saves the output to FILE
     --config=JSON  --cfg          Config definitions (multiple)
@@ -886,6 +914,92 @@ EndOfUsage
 
 ################ Resources ################
 
+use Encode qw(decode decode_utf8 encode_utf8);
+
+sub ::loadfile {
+    my ( $filename, $options ) = @_;
+
+    my $data;			# slurped file data
+    my $encoded;		# already encoded
+
+    # Gather data from the input.
+    if ( ref($filename) ) {
+	$data = $$filename;
+	$filename = "__STRING__";
+	$encoded++;
+    }
+    elsif ( $filename eq '-' ) {
+	$filename = "__STDIN__";
+	$data = do { local $/; <STDIN> };
+    }
+    else {
+	my $name = $filename;
+	$filename = decode_utf8($name);
+	open( my $fh, '<', $name)
+	  or croak("$filename: $!\n");
+	$data = do { local $/; <$fh> };
+    }
+    $options->{_filesource} = $filename if $options;
+
+    my $name = encode_utf8($filename);
+    if ( $encoded ) {
+	# Nothing to do, already dealt with.
+    }
+
+    # Detect Byte Order Mark.
+    elsif ( $data =~ /^\xEF\xBB\xBF/ ) {
+	warn("$name is UTF-8 (BOM)\n") if $options->{debug};
+	$data = decode( "UTF-8", substr($data, 3) );
+    }
+    elsif ( $data =~ /^\xFE\xFF/ ) {
+	warn("$name is UTF-16BE (BOM)\n") if $options->{debug};
+	$data = decode( "UTF-16BE", substr($data, 2) );
+    }
+    elsif ( $data =~ /^\xFF\xFE\x00\x00/ ) {
+	warn("$name is UTF-32LE (BOM)\n") if $options->{debug};
+	$data = decode( "UTF-32LE", substr($data, 4) );
+    }
+    elsif ( $data =~ /^\xFF\xFE/ ) {
+	warn("$name is UTF-16LE (BOM)\n") if $options->{debug};
+	$data = decode( "UTF-16LE", substr($data, 2) );
+    }
+    elsif ( $data =~ /^\x00\x00\xFE\xFF/ ) {
+	warn("$name is UTF-32BE (BOM)\n") if $options->{debug};
+	$data = decode( "UTF-32BE", substr($data, 4) );
+    }
+
+    # No BOM, did user specify an encoding?
+    elsif ( $options->{encoding} ) {
+	warn("$name is ", $options->{encoding}, " (--encoding)\n")
+	  if $options->{debug};
+	$data = decode( $options->{encoding}, $data, 1 );
+    }
+
+    # Try UTF8, fallback to ISO-8895.1.
+    else {
+	my $d = eval { decode( "UTF-8", $data, 1 ) };
+	if ( $@ ) {
+	    warn("$name is ISO-8859.1 (assumed)\n") if $options->{debug};
+	    $data = decode( "iso-8859-1", $data );
+	}
+	else {
+	    warn("$name is UTF-8 (detected)\n") if $options->{debug};
+	    $data = $d;
+	}
+    }
+
+    # Split in lines;
+    my @lines;
+    $data =~ s/^\s+//s;
+    # Unless empty, make sure there is a final newline.
+    $data .= "\n" if $data =~ /.(?!\r\n|\n|\r)\Z/;
+    # We need to maintain trailing newlines.
+    push( @lines, $1 ) while $data =~ /(.*)(?:\r\n|\n|\r)/g;
+
+    return \@lines;
+}
+
+
 sub ::findlib {
     my ( $file ) = @_;
 
@@ -995,6 +1109,9 @@ L<https://github.com/sciurius/chordpro>.
 Please report any bugs or feature requests to the GitHub issue tracker,
 L<https://github.com/sciurius/chordpro/issues>.
 
+A user community discussing ChordPro can be found at
+L<https://groups.google.com/forum/#!forum/chordpro>.
+
 =head1 LICENSE
 
 Copyright (C) 2010,2017 Johan Vromans,
