@@ -367,35 +367,12 @@ sub generate_song {
     my $chorddiagrams = sub {
 	my ( $chords, $show ) = @_;
 
-	if ( $chords ) {
-	    my $vsp = chordgrid_vsp( undef, $ps );
-	    my $hsp = chordgrid_hsp( undef, $ps );
-	    my $h = int( ( $ps->{__rightmargin}
-			   - $ps->{__leftmargin}
-			   + $ps->{diagrams}->{hspace}
-			   * $ps->{diagrams}->{width} ) / $hsp );
-	    my @chords = @$chords;
-	    while ( @chords ) {
-		my $x = $x;
-		$checkspace->($vsp);
-		$pr->show_vpos( $y, 0 ) if DEBUG_SPACING;
-
-		for ( 1..$h ) {
-		    last unless @chords;
-		    my $t = chordgrid( getchordinfo(shift(@chords)), $x, $y, $ps );
-		    redo unless $t;
-		    $x += $hsp;
-		}
-
-		$y -= $vsp;
-		$pr->show_vpos( $y, 1 ) if DEBUG_SPACING;
-	    }
-	    return;
-	}
-
 	my @chords;
-	if ( $s->{chords} ) {
-	    foreach ( @{ $s->{chords}->{chords} } ) {
+	$chords = $s->{chords}->{chords}
+	  if !defined($chords) && $s->{chords};
+	$show //= $ps->{diagrams}->{show};
+	if ( $chords ) {
+	    foreach ( @$chords ) {
 		my $i = getchordinfo($_);
 		push( @chords, $i ) if $i;
 	    }
@@ -413,7 +390,7 @@ sub generate_song {
 
 	# If chord diagrams are to be printed in the right column, put
 	# them on the first page.
-	if ( $ps->{diagrams}->{show} eq "right" && $class <= 1 ) {
+	if ( $show eq "right" && $class <= 1 ) {
 	    my $vsp = chordgrid_vsp( undef, $ps );
 
 	    my $v = int( ( $ps->{_margintop} - $ps->{marginbottom} ) / $vsp );
@@ -441,7 +418,7 @@ sub generate_song {
 		$y -= $vsp;
 	    }
 	}
-	elsif ( $ps->{diagrams}->{show} eq "top" && $class <= 1 ) {
+	elsif ( $show eq "top" && $class <= 1 ) {
 
 	    my $ww = ( $ps->{_marginright} - $ps->{_marginleft} );
 
@@ -466,7 +443,7 @@ sub generate_song {
 	    }
 	    $ps->{_top} = $y;
 	}
-	elsif ( $ps->{diagrams}->{show} eq "bottom" && $class <= 1 && $col == 0 ) {
+	elsif ( $show eq "bottom" && $class <= 1 && $col == 0 ) {
 
 	    my $ww = ( $ps->{_marginright} - $ps->{_marginleft} );
 
@@ -500,9 +477,7 @@ sub generate_song {
 		$pr->show_vpos( $y, 1 ) if DEBUG_SPACING;
 	    }
 	}
-	elsif ( $ps->{diagrams}->{show} eq "below" ) {
-
-	    local $::config->{diagrams}->{show} = $show; # ###WHY?
+	elsif ( $show eq "below" ) {
 
 	    my $vsp = chordgrid_vsp( undef, $ps );
 	    my $hsp = chordgrid_hsp( undef, $ps );
@@ -813,7 +788,7 @@ sub generate_song {
 	}
 
 	if ( $elt->{type} eq "diagrams" ) {
-	    warn("*** CANNOT HAPPEN - type = diagrams ***");
+ 	    $chorddiagrams->( $elt->{chords}, "below" );
 	    next;
 	}
 
@@ -856,9 +831,6 @@ sub generate_song {
 		    delete( $ps->{fonts}->{$1}->{color} );
 		}
 	    }
-	    elsif ( $elt->{name} eq "tag" ) {
-		$i_tag = $elt->{value};
-	    }
 	    next;
 	}
 
@@ -883,6 +855,9 @@ sub generate_song {
 		$cells += $grid_margin->[1] = $v[3] if $v[3];
 		$grid_margin->[2] = $cells;
 	    }
+	    elsif ( $elt->{name} eq "tag" ) {
+		$i_tag = $elt->{value};
+	    }
 	    # Arbitrary config values.
 	    elsif ( $elt->{name} =~ /^pdf\.(.+)/ ) {
 		my @k = split( /[.]/, $1 );
@@ -908,7 +883,7 @@ sub generate_song {
     }
 
     if ( $ps->{diagrams}->{show} eq "below" ) {
-	$chorddiagrams->();
+	$chorddiagrams->( undef, "below");
     }
 
     return $thispage - $startpage + 1;
@@ -1587,7 +1562,8 @@ sub chordgrid {
     $y -= $font->{size} * $ps->{spacing}->{chords} + $dot/2 + $lw;
 
     if ( $info->{base} > 0 ) {
-	my $i = @Roman[$info->{base}] . "  ";
+	# my $i = @Roman[$info->{base}] . "  ";
+	my $i = sprintf("%d  ", 1+$info->{base});
 	$pr->setfont( $ps->{fonts}->{diagram_base}, $gh );
 	$pr->text( $i, $x-$pr->strwidth($i), $y-$gh/2,
 		   $ps->{fonts}->{diagram_base}, $gh );
